#pragma rtGlobals=3
#pragma IgorVersion = 8
#pragma version = 2.10
#pragma hide=1
#pragma IndependentModule = DemoSetupTools

// updater headers
Static Constant kProjectID=21825
Static Strconstant ksShortTitle="Demo Setup Tools"
Static Constant k_version=2.10

Static StrConstant k_pckgfolder = "root:Packages:DemoSetupTools"

// Demo Setup Tools
// 2023-03-27 - allows for graph above, panel below layout
//						(pmode=1 optional switch in send to layout code)
//					- allows for no external procedure file
//						(exprfile=0 optional switch in convert_toDemo function call)
//					- procname can be blank to use experiment name
//					- notestr is now required input to local functions
// 2023-03-14 - includes Convert_toDemo and Lock_Demo
// 2022-09-21 - allows for all graphs to notebook (allgraphs option)
// 2022-03-14 - allows for a panel with no graph
// 2022-02-22
// 2022-02-12 jjw
// hide all Igor menus
// protect the graph and panel
// show a demo menu
// send results to notebook
// send results to layout
// requires Igor Pro 8 or greater

// US letter page in landscape with 1/4 in margin
Static Constant spgwidth = 792
Static Constant spgheight = 612
Static Constant spgmargin = 18
// graph takes this fraction of layout page
Static Constant sgfrac = 0.75

// *** Make Demo ***

Static Function AfterCompiledHook()
	
	NVAR/Z/SDFR=root: demomode
	
	ShowIgorMenus
	if (!NVAR_exists(demomode))
		initialize_Demo(0)
	else
		initialize_Demo(demomode)
	endif
	
	return 0
end

// initialize

Static Function initialize_Demo(variable how)
	
	NVAR/Z/SDFR=root: UIControl, pgwidth, pgheight, pgmargin, gfrac, plock

	if (!DataFolderExists(k_pckgfolder))
		NewDataFolder/O/S root:Packages
		NewDataFolder/O/S DemoSetupTools
		variable/G guic, gpgwidth, gpgheight, gpgmargin, ggfrac, onlypanel, gplock
		string/G ggname, gpname
	else
		SetDataFolder $k_pckgfolder
		// update after 1.20 for only panel and plock
		variable/G onlypanel, gplock
	endif
	
	DFREF pf = $k_pckgfolder
	NVAR/SDFR=pf guic, gpgwidth, gpgheight, gpgmargin, ggfrac, onlypanel, gplock
	SVAR/SDFR=pf ggname, gpname
	
	ggname = ""
	gpname = ""
	
	if (NVAR_exists(UIControl))
		guic = UIControl
	else
		guic = 4095
	endif
		
	if (NVAR_exists(pgwidth))
		gpgwidth = pgwidth
	else
		gpgwidth = spgwidth
	endif
		
	if (NVAR_exists(pgheight))
		gpgheight = pgheight
	else
		gpgheight = spgheight
	endif
		
	if (NVAR_exists(pgmargin))
		gpgmargin = pgmargin
	else
		gpgmargin = spgmargin
	endif
		
	if (NVAR_exists(gfrac))
		ggfrac = gfrac
	else
		ggfrac = sgfrac
	endif
		
	if (NVAR_exists(plock))
		gplock = plock
	else
		gplock = 1
	endif
	
	ggname = WinName(0,1,1)
	gpname = WinName(0,64,1)
	
	if (strlen(gpname)==0)
		gpname = StringFromList(0,ChildWindowList(ggname))
	endif

	if (strlen(ggname) == 0)
		if (strlen(gpname) == 0)
			SetDataFolder root:
			return -1
		endif
		onlypanel = 1
	endif
		
	SetDataFolder root:
	SVAR/Z gpassword
	string lpassword, hpassword
	
	if (how==1)
		HideIgorMenus
		if (strlen(ggname)!=0)
			ModifyGraph/W=$ggname UIControl=guic
		endif
		if (strlen(gpname)!=0)
			ModifyPanel/W=$gpname noEdit=1, fixedSize=1
		endif
	else
		if (SVAR_exists(gpassword))
			Prompt lpassword, "What password to open me?"
			DoPrompt "Password", lpassword
			if (v_flag)
				variable/G demomode = 1
				HideIgorMenus
				return 0
			endif
			hpassword = Hash(lpassword,1)
			if (cmpstr(hpassword,gpassword) != 0)
				DoAlert 0, "Wrong password"
				variable/G demomode = 1
				HideIgorMenus
				return 0
			endif
		endif
		ShowIgorMenus
		if (strlen(ggname)!=0)
			ModifyGraph/W=$ggname UIControl=0
		endif
		if (strlen(gpname)!=0)
			ModifyPanel/W=$gpname noEdit=0, fixedSize=0
		endif
		killstrings/Z gpassword
	endif
	
	return 0
end

// *** Layout ***

// create layout
Function layout_FrontSet(string notestr, [variable pmode])
	
	DFREF pf = root:Packages:DemoSetupTools
	SVAR/SDFR=pf ggname, gpname
	
	if (ParamIsDefault(pmode))
		pmode = 0
	endif
	if (strlen(notestr) == 0)
		layout_results(ggname, gpname, pmode)
	else
		layout_results(ggname, gpname, pmode, notestr = notestr)
	endif
	
	return 0
end

// set gfill = 1 to have graph fill its region
Static Function layout_results(string graphname, string panelname, variable pmode, [string notestr, variable gfill])

	DFREF pf = root:Packages:DemoSetupTools
	// globals
	NVAR/SDFR=pf gpgwidth, gpgheight, gpgmargin, ggfrac, onlypanel
	
	// graph and panel actual sizes
	variable gwidth, gheight, pwidth, pheight
	// graph and panel display space widths on page
	variable grwidth, prwidth
	// graph and panel aspect ratios
	variable gratio, pratio, wthratio
	// offset left to panel or below to panel
	variable toffset, tgwidth, tgheight
	variable gonly = 0
	variable gpgwidtho, gpgheighto
	string dpstr
	
	if (pmode)
		gpgwidtho = gpgwidth
		gpgheighto = gpgheight
		gpgwidth = gpgheighto
		gpgheight = gpgwidtho
	endif
	
	if (onlypanel)
		
		// panel only
		// sizes of regions for appending
		
		grwidth = 0.9*(gpgwidth-2*gpgmargin)
		prwidth = 0.1*(gpgwidth-3*gpgmargin)
		
		gratio = grwidth/(gpgheight-2*gpgmargin)
		pratio = prwidth/(gpgheight-2*gpgmargin-4*gpgmargin)
		
		// validity checks	 and window sizes
		
		SavePICT/SNAP=1/O/WIN=$panelname/P=_PictGallery_/E=-5 as "PanelSnapShot"
		GetWindow/Z $panelname wsize
		pwidth = V_right - V_left; pheight = V_bottom - V_top
	
	elseif (strlen(panelname)!=0)

		// graph & panel		
		// sizes of regions for appending
		
		if (pmode)
			grwidth = (gpgwidth-2*gpgmargin)
			prwidth = (gpgwidth-8*gpgmargin)			
		else
			grwidth = ggfrac*(gpgwidth-2*gpgmargin)
			prwidth = (1 - ggfrac)*(gpgwidth-3*gpgmargin)
		endif
		
		gratio = grwidth/(gpgheight-2*gpgmargin)
		pratio = prwidth/(gpgheight-2*gpgmargin-4*gpgmargin)
		
		// validity checks	 and window sizes
		
		DoWindow/F $graphname
		if (v_flag == 0)
			return -1
		endif	
		GetWindow/Z $graphname wsize
		gwidth = V_right - V_left; gheight = V_bottom - V_top
		//print gwidth, gheight
		
		SavePICT/SNAP=1/O/WIN=$panelname/P=_PictGallery_/E=-5 as "PanelSnapShot"
		GetWindow/Z $panelname wsize
		pwidth = V_right - V_left; pheight = V_bottom - V_top

	else
		
		// graph only		
		// sizes of regions for appending
		
		gonly = 1
		grwidth = (gpgwidth-2*gpgmargin)
		prwidth = 0
		
		gratio = grwidth/(gpgheight-2*gpgmargin)
		pratio = 0
		
		// validity checks	 and window sizes
		
		DoWindow/F $graphname
		if (v_flag == 0)
			return -1
		endif	
		GetWindow/Z $graphname wsize
		gwidth = V_right - V_left; gheight = V_bottom - V_top
	endif
		
	// strings for stamp
	
	string enstr, strdt, strsys, strprbx
	
	enstr = IgorInfo(1) + "\r"
	strdt = date() + " " + time()
	strsys = IgorInfo(7) + " : " + IgorInfo(2)	
	strprbx = "\\Z10\\JR" + enstr + strsys
	strprbx += "\r" + strdt
	if (!ParamIsDefault(notestr))
		strprbx += "\rNotes: " + notestr
	endif
	
	// remove window to refresh
	
	KillWindow/Z SnapshotResultsLayout
	
	NewLayout/K=1/W=(30,50,600,500) as "Results Layout"
	DoWindow/C SnapshotResultsLayout
	LayoutPageAction size=(gpgwidth,gpgheight),margins=(gpgmargin,gpgmargin,gpgmargin,gpgmargin)
	ModifyLayout mag=0.5
	
	if (onlypanel)
		
		// append panel and text stamp
		
		SetDrawLayer ProgBack
		wthratio = pwidth/pheight
		if (wthratio > gratio)
			//print "wide panel", pwidth, pheight, wthratio, pratio
			//toffset = pratio*pheight - 2*gpgmargin
			toffset = 2*gpgmargin
			pratio = prwidth/pwidth
		else
			//print "tall panel", pwidth, pheight
			toffset = gpgmargin
			pratio = (gpgheight-2*gpgmargin-4*gpgmargin)/pheight
		endif		
		sprintf dpstr, "DrawPICT %f,%f,%f,%f,PanelSnapShot",gpgmargin,toffset,pratio,pratio
		Execute/Q/Z dpstr
		//DrawPICT grwidth+pgmargin,toffset,pratio,pratio,PanelSnapShot	
		DrawText 1.5*gpgmargin+pratio*pwidth, gpgheight-1.5*gpgmargin, strprbx
		SetDrawLayer UserFront

	else
	
		// append graph
		
		if (ParamIsDefault(gfill))
			wthratio = gwidth/gheight
			if (wthratio > gratio)
				// too wide, shrink height by ratio
				tgheight = gheight*(grwidth/gwidth)
				toffset = 2*gpgmargin
				tgwidth = tgheight*gratio
				AppendLayoutObject/F=0/T=0/R=(gpgmargin,toffset,grwidth+gpgmargin,tgheight+toffset) Graph $graphname
				//print "too wide", wthratio, grwidth/tgheight
			else
				// too tall, shrink width by ratio
				toffset = gpgmargin
				tgwidth = gwidth*(gpgheight-2*gpgmargin)/gheight
				tgheight = tgwidth/gratio
				AppendLayoutObject/F=0/T=0/R=(gpgmargin,toffset,tgwidth+gpgmargin,gpgheight-gpgmargin) Graph $graphname
				//print "too tall", wthratio, tgwidth/(pgheight-2*pgmargin)
			endif
		else
			AppendLayoutObject/F=0/T=0/R=(gpgmargin,gpgmargin,grwidth+gpgmargin,gpgheight-2*gpgmargin) Graph $graphname
		endif
		
		// append panel and text stamp
		
		SetDrawLayer ProgBack
		if (pmode)
			if (!gonly)
				wthratio = pwidth/pheight
				if (wthratio > pratio)
					//print "wide panel", pwidth, pheight, wthratio, pratio
					//toffset = pratio*pheight - 2*gpgmargin
					toffset = tgheight + 3*gpgmargin
					pratio = prwidth/pwidth
				else
					//print "tall panel", pwidth, pheight
					toffset = tgheight + 3*gpgmargin
					pratio = (gpgheight-2*gpgmargin-4*gpgmargin)/pheight
				endif		
				sprintf dpstr, "DrawPICT %f,%f,%f,%f,PanelSnapShot",2*gpgmargin,toffset,pratio,pratio
				Execute/Q/Z dpstr
				SetDrawEnv textxjust=2
				DrawText gpgwidth - 2*gpgmargin, gpgheight-1.5*gpgmargin, strprbx
			else
				SetDrawEnv textxjust=2
				DrawText gpgwidth - 2*gpgmargin, gpgheight-1.5*gpgmargin, strprbx
			endif
		else
			if (!gonly)
				wthratio = pwidth/pheight
				if (wthratio > pratio)
					//print "wide panel", pwidth, pheight, wthratio, pratio
					//toffset = pratio*pheight - 2*gpgmargin
					toffset = 2*gpgmargin
					pratio = prwidth/pwidth
				else
					//print "tall panel", pwidth, pheight
					toffset = gpgmargin
					pratio = (gpgheight-2*gpgmargin-4*gpgmargin)/pheight
				endif		
				sprintf dpstr, "DrawPICT %f,%f,%f,%f,PanelSnapShot",grwidth+gpgmargin,toffset,pratio,pratio
				Execute/Q/Z dpstr
				//SetDrawEnv textxjust=2
				DrawText grwidth+gpgmargin, gpgheight-1.5*gpgmargin, strprbx
			else
				//SetDrawEnv textxjust=2
				DrawText 1.5*gpgmargin, gpgheight-1.5*gpgmargin, strprbx
			endif
		endif
		SetDrawLayer UserFront
	endif

	if (pmode)
		gpgwidth = gpgwidtho
		gpgheight = gpgheighto
	endif

	return 0
end

// *** Notebook ***

// create notebook
Function notebook_FrontSet(string notestr, [variable allgraphs])
	
	DFREF pf = root:Packages:DemoSetupTools
	SVAR/SDFR=pf ggname, gpname
	
	string ggnamelist
	
	if (ParamIsDefault(allgraphs) || allgraphs == 0)
		ggnamelist = ggname
	else
		ggnamelist = WinList("*",";","WIN:1")
	endif
	
	if (strlen(notestr)==0)
		notebook_results(gpname, ggnamelist)
	else
		notebook_results(gpname, ggnamelist, notestr = notestr)
	endif
	
	return 0
end

// put results in notebook
Static Function notebook_results(string panelname, string gnamelist, [string notestr])

	string ntbkstr, ntbkName, graphname
	variable ic
	
	ntbkstr = IgorInfo(1) + "_ntbk" 
	
	ntbkName = CleanupName(ntbkstr,0)
	
	if (strlen(WinList(ntbkName,";","WIN:16")) == 0)
		NewNotebook/F=1/N=$ntbkName as ntbkstr
		create_NtbkRulers(ntbkName)
		put_FirstHeaders(ntbkName)
	endif
	
	// put information

	if (ParamIsDefault(notestr))
		put_Info(ntbkName)
	else
		put_Info(ntbkName, notestr = notestr)
	endif
	
	// put the graph and panel

	if (strlen(panelname)!=0)
		put_thePanel(ntbkName, panelname)
	endif
	if (strlen(graphname)!=0)
		for (ic=0;ic<ItemsInList(gnamelist);ic+=1)
			graphname = StringfromList(ic,gnamelist)
			put_theGraph(ntbkName, graphname)
		endfor
	endif
	
	return 0
end

// put graph
Static Function put_theGraph(string theNotebook, string thegraph)

	string gtitle, gname
	variable wsize, gwidth
	
	GetWindow/Z $thegraph, wtitle
	gtitle = thegraph + " - " +  s_value
	gname = "\rGraph: " + gtitle + "\r"
	
	Notebook $theNotebook, scaling={50,50}, picture={$thegraph,-5,1}
	Notebook $theNotebook, text = gname
	return 0
end

Static Function put_thePanel(string theNotebook, string thepanel)

	string gtitle, gname
	variable wsize, gwidth
	
	GetWindow/Z $thepanel, wtitle
	gtitle = thepanel + " - " +  s_value
	gname = "\rPanel: " + gtitle + "\r"
	SavePICT/SNAP=1/O/WIN=$thepanel/P=_PictGallery_/E=-5 as thepanel
	
	Notebook $theNotebook, scaling={50,50}, picture={$thepanel,-5,1}
	Notebook $theNotebook, text = gname
	return 0
end

Static Function put_Info(string theNotebook, [string notestr])

	string tstr
	
	Notebook $theNotebook ruler=Subheading
	Notebook $theNotebook text="## Information\r"
	Notebook $theNotebook ruler=Normal
	tstr = "#date " + date() + " #time " + time() + "\r"
	if (!ParamIsDefault(notestr))
		tstr += "#note " + notestr + "\r"
	endif
	Notebook $theNotebook text = tstr	
	return 0
end

Static Function Create_NtbkRulers(string theNtbk)
	
	Notebook $theNtbk newRuler=Heading, spacing={12,6,0}, rulerDefaults={"",14,1,(0,0,0)}
	Notebook $theNtbk newRuler=Subheading, spacing={6,6,0}, rulerDefaults={"",12,2,(0,0,0)}
	Notebook $theNtbk Ruler=Normal, spacing={0,0,0}, rulerDefaults={"",12,0,(0,0,0)}
	return 0
end

Static Function put_FirstHeaders(string theNotebook)

	string tstr
	
	Notebook $theNotebook ruler=Heading
	tstr = "# Start of Notebook\r"
	Notebook $theNotebook text = tstr
	
	Notebook $theNotebook ruler=Normal
	tstr = "#date " + date() + "\r"
	Notebook $theNotebook text = tstr
	
	Notebook $theNotebook ruler=Heading
	tstr = "# Experiment Information\r"
	Notebook $theNotebook text = tstr
		
	Notebook $theNotebook ruler=Normal
	tstr = "Project Name: " + IgorInfo(1)
	tstr += "\rIgor Version Information: " + IgorInfo(3)
	tstr += "\rUser Name: " + IgorInfo(7) + "\r"
	Notebook $theNotebook text = tstr

	return 0
end

// convert the current experiment to a demo
Function Convert_toDemo([string procname, variable exprfile])

	SVAR/Z/SDFR=root: gpckgproc
	NVAR/Z/SDFR=$k_pckgfolder gplock
	
	string currexpname, saveexpname
	string demoproc = "Demo Setup Tools", demoprocfile, pckgproc, pckgprocfile
	string fcmdstr, acmdStr = "AdoptFiles/W=$", dcmdstr = "DELETEINCLUDE \""
	string prstr = "What name should be used for the working copy experiment?"
	string pname = ""
//	string psstr = "What password should be used to open this to full mode (DEFAULT: openme)?"
	
	NVAR/Z/SDFR=root: demomode
	
	if (!NVAR_exists(demomode))
		initialize_Demo(0)
	endif
	
	currexpname = IgorInfo(1)
	
	SetDataFolder root:
	
	if (!SVAR_Exists(gpckgproc))
		string/G gpckgproc = ""
	endif
	if (ParamIsDefault(exprfile))
		exprfile = 0
	endif
	if (ParamIsDefault(procname))
		if (strlen(gpckgproc) == 0)
			procname = currexpname
			gpckgproc = currexpname
		else
			procname = gpckgproc
		endif
	else
		exprfile=1
	endif
	
	pckgprocfile = "\"" + gpckgproc + ".ipf\""
	demoprocfile = "\"" + demoproc + ".ipf\""
	
	do
		saveexpname = currexpname	 + " WC"
		Prompt saveexpname, prstr
		DoPrompt "Working Copy Experiment Name", saveexpname
		if (v_flag)
			return 0
		endif
		if (cmpstr(saveexpname,currexpname) == 0)
			DoAlert 0, "Working copy experiment name cannot be the same as the current experiment name"
		else
			break
		endif
	while(1)
	
	saveexpname += ".pxp"
	
	SaveExperiment/C as saveexpname
	
	// create global
	
	variable/G demomode=0
	
	switch(gplock)
		case -1:
			pname = WinList("*","","WIN:64")
			print "unlocking panel " + pname
			ModifyPanel/W=$pname fixedSize=0, noEdit=0
			break
		case 1:
			pname = WinList("*","","WIN:64")
			print "locking panel " + pname
			ModifyPanel/W=$pname fixedSize=1, noEdit=1
			break
	endswitch
	
	if (exprfile)
		fcmdStr = dcmdStr + gpckgproc + "\""
		print "deleting included file " + fcmdStr
		Execute/P fcmdStr
	endif
	
	fcmdStr = acmdStr + pckgprocfile
	Execute/P/Q fcmdStr
	
	fcmdStr = "DELETEINCLUDE \"" + demoproc + "\""
	print "deleting included file " + fcmdStr
	Execute/P fcmdStr
	
	fcmdStr = acmdStr + demoprocfile
	Execute/P/Q fcmdStr

	demomode=1	
	
	//Execute/P/Q "COMPILEPROCEDURES"
	
	return 0
end

// lock the current experiment when in demo mode
Function lock_Demo([string password])
	
	SetDataFolder root:
	SVAR/Z gpassword
	
	if (!ParamIsDefault(password))
		if (!SVAR_exists(gpassword))
			string/G gpassword
		endif
		SVAR gpassword
		gpassword = HASH(password,1)
		print "Demo locked with password ", password, " HASHED as ", gpassword
	else
		if (!SVAR_exists(gpassword))
			DoAlert 0, "No global password. Use password=... at input"
			return 0
		else
			SVAR gpassword
		endif	
		print "Demo locked with unknown password HASHED as ", gpassword
	endif
	
	return 0
end